/**********************************************************************
 *
 * ResultItemDocument.java -- a result in the Phind applet
 *
 * Copyright 2000 Gordon W. Paynter
 * Copyright 2000 The New Zealand Digital Library Project
 *
 * A component of the Greenstone digital library software
 * from the New Zealand Digital Library Project at the
 * University of Waikato, New Zealand.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *********************************************************************/

/*********************************************************************

This class is used in the Phind java applet (Phind.java).

A result item holds the data describing a single line returned from a phind
query.  The complex functionality of phrase and URL items is implemented in
subclasses; the base class handles simpler cases like the "get more phrases" 
marker.

**********************************************************************/


public class ResultItem {

    // There are several kinds of ResultItem
    //   1. a phrase (expansion)
    //   2. a "get more phrases" marker
    //   3. a document (or URL)
    //   4. a "get more documents" marker
    //   5. a thesaurus link
    //   6. a "get more thesaurus links" marker

    final static int linkItem = 1;
    final static int moreLinks = 2;
    final static int phraseItem = 3;
    final static int morePhrases = 4;
    final static int documentItem = 5;
    final static int moreDocuments = 6;
    final static int unknownResultItem = 9;

    // A text string describing the item
    String text = "Generic ResultItem";

    // The kind of the item
    int kind = unknownResultItem;

    // The frequency of the item
    int frequency = 0;


    // Create a blank ResultItem
    ResultItem() {
	text = "Unknown result type";
	kind = unknownResultItem;
	frequency = 0;
    }

    // Create a new ResultItem of some given kind
    ResultItem(int newKind) {
	kind = newKind;
	frequency = 0;

	if (kind == moreLinks) {
	    text = "get more thesaurus links";
	} else if (kind == morePhrases) {
	    text = "get more phrases";
	} else if (kind == moreDocuments) {
	    text = "get more documents";
	} else {
	    text = "Unknown result type";
	}
    }

    // Test the type of a ResultItem
    public boolean isLink() { return false; }
    public boolean isMoreLinks() { return (kind == moreLinks); }
    public boolean isPhrase() { return false; }
    public boolean isMorePhrases() { return (kind == morePhrases); }
    public boolean isDocument() { return false; }
    public boolean isMoreDocuments() { return (kind == moreDocuments); }

    // Return the bare text of the item
    public String toString() { return(text.trim()); }

    // Return the text of the item components
    public String mainText() { return text; }
    public String freqText() { 
	if (frequency > 0) { return Integer.toString(frequency); } 
	else { return ""; }
    }
    public String docsText() { return ""; }
    public String prefixText() { return ""; }
    public String suffixText() { return ""; }
    public String hiddenText() { return ""; }
}

